@@ -1,5 +1,4 @@
-/* $Id$ */
-/*-
+/*
  * Copyright (c) 2006 Benedikt Meurer <benny@xfce.org>.
  *
  * This library is free software; you can redistribute it and/or
@@ -13,9 +12,9 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA 02110-1301 USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,6 +31,7 @@
 #include <libxfce4ui/xfce-titled-dialog.h>
 #include <libxfce4ui/libxfce4ui-private.h>
 #include <libxfce4ui/libxfce4ui-alias.h>
+#include <libxfce4ui/xfce-gtk-extensions.h>
 
 
 #define XFCE_TITLED_DIALOG_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), XFCE_TYPE_TITLED_DIALOG, XfceTitledDialogPrivate))
@@ -107,8 +107,8 @@ xfce_titled_dialog_class_init (XfceTitledDialogClass *klass)
 
   /* connect additional key bindings to the GtkDialog::close action signal */
   binding_set = gtk_binding_set_by_class (klass);
-  gtk_binding_entry_add_signal (binding_set, GDK_w, GDK_CONTROL_MASK, "close", 0);
-  gtk_binding_entry_add_signal (binding_set, GDK_W, GDK_CONTROL_MASK, "close", 0);
+  gtk_binding_entry_add_signal (binding_set, GDK_KEY_w, GDK_CONTROL_MASK, "close", 0);
+  gtk_binding_entry_add_signal (binding_set, GDK_KEY_W, GDK_CONTROL_MASK, "close", 0);
 }
 
 
@@ -118,40 +118,47 @@ xfce_titled_dialog_init (XfceTitledDialog *titled_dialog)
 {
   GtkWidget *line;
   GtkWidget *vbox;
+  GtkWidget *content_area;
 
   /* connect the private data */
   titled_dialog->priv = XFCE_TITLED_DIALOG_GET_PRIVATE (titled_dialog);
 
   /* remove the main dialog box from the window */
-  g_object_ref (G_OBJECT (GTK_DIALOG (titled_dialog)->vbox));
-  gtk_container_remove (GTK_CONTAINER (titled_dialog), GTK_DIALOG (titled_dialog)->vbox);
+  content_area = gtk_dialog_get_content_area (GTK_DIALOG (titled_dialog));
+  g_object_ref (G_OBJECT (content_area));
+  gtk_container_remove (GTK_CONTAINER (titled_dialog), content_area);
 
   /* add a new vbox w/o border to the main window */
+#if GTK_CHECK_VERSION (3, 0, 0)
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
   vbox = gtk_vbox_new (FALSE, 0);
+#endif
   gtk_container_add (GTK_CONTAINER (titled_dialog), vbox);
   gtk_widget_show (vbox);
 
   /* add the heading to the window */
-  titled_dialog->priv->heading = _xfce_heading_new ();
+  titled_dialog->priv->heading = g_object_new (XFCE_TYPE_HEADING, NULL);
   gtk_box_pack_start (GTK_BOX (vbox), titled_dialog->priv->heading, FALSE, FALSE, 0);
   gtk_widget_show (titled_dialog->priv->heading);
 
   /* add the separator between header and content */
+#if GTK_CHECK_VERSION (3, 0, 0)
+  line = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+#else
   line = gtk_hseparator_new ();
+#endif
   gtk_box_pack_start (GTK_BOX (vbox), line, FALSE, FALSE, 0);
   gtk_widget_show (line);
 
   /* add the main dialog box to the new vbox */
-  gtk_box_pack_start (GTK_BOX (vbox), GTK_DIALOG (titled_dialog)->vbox, TRUE, TRUE, 0);
-  g_object_unref (G_OBJECT (GTK_DIALOG (titled_dialog)->vbox));
+  gtk_box_pack_start (GTK_BOX (vbox), content_area, TRUE, TRUE, 0);
+  g_object_unref (G_OBJECT(content_area));
 
   /* make sure to update the heading whenever one of the relevant window properties changes */
   g_signal_connect (G_OBJECT (titled_dialog), "notify::icon", G_CALLBACK (xfce_titled_dialog_update_heading), NULL);
   g_signal_connect (G_OBJECT (titled_dialog), "notify::icon-name", G_CALLBACK (xfce_titled_dialog_update_heading), NULL);
   g_signal_connect (G_OBJECT (titled_dialog), "notify::title", G_CALLBACK (xfce_titled_dialog_update_heading), NULL);
-
-  /* initially update the heading properties */
-  xfce_titled_dialog_update_heading (titled_dialog);
 }
 
 
@@ -220,11 +227,11 @@ xfce_titled_dialog_close (GtkDialog *dialog)
   GdkEvent *event;
 
   /* verify that the dialog is realized */
-  if (G_LIKELY (GTK_WIDGET_REALIZED (dialog)))
+  if (G_LIKELY (gtk_widget_get_realized (GTK_WIDGET (dialog))))
     {
       /* send a delete event to the dialog */
       event = gdk_event_new (GDK_DELETE);
-      event->any.window = g_object_ref (GTK_WIDGET (dialog)->window);
+      event->any.window = g_object_ref (gtk_widget_get_window(GTK_WIDGET (dialog)));
       event->any.send_event = TRUE;
       gtk_main_do_event (event);
       gdk_event_free (event);
@@ -287,7 +294,6 @@ xfce_titled_dialog_new_with_buttons (const gchar    *title,
   /* allocate the dialog */
   dialog = g_object_new (XFCE_TYPE_TITLED_DIALOG,
                          "destroy-with-parent", ((flags & GTK_DIALOG_DESTROY_WITH_PARENT) != 0),
-                         "has-separator", ((flags & GTK_DIALOG_NO_SEPARATOR) == 0),
                          "modal", ((flags & GTK_DIALOG_MODAL) != 0),
                          "title", title,
                          NULL);
@@ -311,6 +317,81 @@ xfce_titled_dialog_new_with_buttons (const gchar    *title,
 
 
 
+/**
+ * xfce_titled_dialog_new_with_mixed_buttons:
+ * @title                  : title of the dialog, or %NULL.
+ * @parent                 : transient parent window of the dialog, or %NULL.
+ * @flags                  : from #GtkDialogFlags.
+ * @first_button_icon_name : icon name to go in first, or "" for no icon.
+ * @first_button_text      : text to go in first, or %NULL.
+ * @...                    : response ID for the first button, then additional buttons, ending with %NULL.
+ *
+ * Creates an #XfceTitledDialog using xfce_gtk_button_new_mixed. This allows
+ * the buttons to use an optional named or stock icon.
+ *
+ * Return value: the newly allocated #XfceTitledDialog.
+ **/
+GtkWidget*
+xfce_titled_dialog_new_with_mixed_buttons (const gchar    *title,
+                                           GtkWindow      *parent,
+                                           GtkDialogFlags  flags,
+                                           const gchar    *first_button_icon_name,
+                                           const gchar    *first_button_text,
+                                           ...)
+{
+  const gchar *icon_name;
+  const gchar *button_text;
+  GtkWidget   *dialog;
+  va_list      args;
+  gint         response_id;
+
+  /* allocate the dialog */
+  dialog = g_object_new (XFCE_TYPE_TITLED_DIALOG,
+                         "destroy-with-parent", ((flags & GTK_DIALOG_DESTROY_WITH_PARENT) != 0),
+                         "modal", ((flags & GTK_DIALOG_MODAL) != 0),
+                         "title", title,
+                         NULL);
+
+  /* set the transient parent (if any) */
+  if (G_LIKELY (parent != NULL))
+    gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
+
+  /* add all additional buttons */
+  icon_name = first_button_icon_name;
+  button_text = first_button_text;
+  va_start (args, first_button_text);
+
+  while (icon_name != NULL)
+    {
+      GtkWidget *button;
+
+      /* response id comes after button text */
+      response_id = va_arg (args, gint);
+
+      /* build our button and add it */
+      button = xfce_gtk_button_new_mixed (icon_name, button_text);
+      gtk_widget_set_can_default (button, TRUE);
+
+      gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, response_id);
+      gtk_widget_show (button);
+
+      /* this is to pickup for the next button.
+       * The pattern is icon_name, button text
+       */
+      icon_name = va_arg (args, const gchar *);
+      if (icon_name)
+        {
+          button_text = va_arg (args, const gchar *);
+        }
+    }
+  va_end (args);
+
+  return dialog;
+}
+
+
+
+
 /**
  * xfce_titled_dialog_get_subtitle:
  * @titled_dialog : a #XfceTitledDialog.
@@ -320,7 +401,7 @@ xfce_titled_dialog_new_with_buttons (const gchar    *title,
  *
  * Return value: the subtitle of @titled_dialog, or %NULL.
  **/
-G_CONST_RETURN gchar*
+const gchar*
 xfce_titled_dialog_get_subtitle (XfceTitledDialog *titled_dialog)
 {
   g_return_val_if_fail (XFCE_IS_TITLED_DIALOG (titled_dialog), NULL);
